@@ -5,12 +5,14 @@
 #' @param B An ordinally weighted bipartite graph, as: (1) an incidence matrix in the form of a matrix or sparse \code{\link{Matrix}}; (2) an edgelist in the form of a three-column dataframe; (3) an \code{\link{igraph}} object.
 #' @param trials integer: the number of bipartite graphs generated to approximate the edge weight distribution. If NULL, the number of trials is selected based on `alpha` (see details)
 #' @param alpha real: significance level of hypothesis test(s)
+#' @param missing.as.zero boolean: should missing edges be treated as edges with zero weight and tested for significance
 #' @param signed boolean: TRUE for a signed backbone, FALSE for a binary backbone (see details)
 #' @param mtc string: type of Multiple Test Correction to be applied; can be any method allowed by \code{\link{p.adjust}}.
 #' @param class string: the class of the returned backbone graph, one of c("original", "matrix", "Matrix", "igraph", "edgelist").
 #'     If "original", the backbone graph returned is of the same class as `B`.
 #' @param narrative boolean: TRUE if suggested text & citations should be displayed.
 #' @param progress boolean: TRUE if the progress of Monte Carlo trials should be displayed.
+#' @param ... optional arguments
 #'
 #' @details
 #' The `osdsm` function compares an edge's observed weight in the projection `B*t(B)` to the distribution of weights
@@ -18,21 +20,17 @@
 #'    approximately the same number of each value. The edges in `B` must be integers, and are assumed to represent an
 #'    ordinal-level measure such as a Likert scale that starts at 0.
 #'
-#' When `signed = FALSE`, a one-tailed test (is the weight stronger) is performed for each edge with a non-zero weight. It
-#'    yields a backbone that perserves edges whose weights are significantly *stronger* than expected in the chosen null
-#'    model. When `signed = TRUE`, a two-tailed test (is the weight stronger or weaker) is performed for each every pair of nodes.
-#'    It yields a backbone that contains positive edges for edges whose weights are significantly *stronger*, and
-#'    negative edges for edges whose weights are significantly *weaker*, than expected in the chosen null model.
-#'    *NOTE: Before v2.0.0, all significance tests were two-tailed and zero-weight edges were evaluated.*
+#' When `signed = FALSE`, a one-tailed test (is the weight stronger?) is performed for each edge. The resulting backbone
+#'    contains edges whose weights are significantly *stronger* than expected in the null model. When `signed = TRUE`, a
+#'    two-tailed test (is the weight stronger or weaker?) is performed for each edge. The resulting backbone contains
+#'    positive edges for those whose weights are significantly *stronger*, and negative edges for those whose weights are
+#'    significantly *weaker*, than expected in the null model.
 #'
 #' The p-values used to evaluate the statistical significance of each edge are computed using Monte Carlo methods. The number of
-#'    `trials` performed affects the precision of these p-values, and the confidence that a given p-value is less than the
-#'    desired `alpha` level. Because these p-values are proportions (i.e., the proportion of times an edge is weaker/stronger
-#'    in the projection of a random bipartite graphs), evaluating the statistical significance of an edge is equivalent to
-#'    comparing a proportion (the p-value) to a known proportion (alpha). When `trials = NULL`, the `power.prop.test` function
-#'    is used to estimate the required number of trials to make such a comparison with a `alpha` type-I error rate, (1-`alpha`) power,
-#'    and when the riskiest p-value being evaluated is at least 5% smaller than `alpha`. When any `mtc` correction is applied,
-#'    for simplicity this estimation is based on a conservative Bonferroni correction.
+#'    `trials` performed affects the precision of these p-values. This precision impacts the confidence that a given edge's p-value
+#'    is less than the desired `alpha` level, and therefore represents a statistically significant edge that should be retained in
+#'    the backbone. When `trials = NULL`, [trials.needed()] is used to estimate the required number of trials to evaluate the
+#'    statistical significance of an edges' p-values.
 #'
 #' @return
 #' If `alpha` != NULL: Binary or signed backbone graph of class `class`.
@@ -62,7 +60,7 @@
 #'             class = "igraph", trials = 100)
 #' plot(bb) #...is sparse with clear communities
 
-osdsm <- function(B, alpha = 0.05, trials = NULL, signed = FALSE, mtc = "none", class = "original", narrative = FALSE, progress = TRUE){
+osdsm <- function(B, alpha = 0.05, trials = NULL, missing.as.zero = FALSE, signed = FALSE, mtc = "none", class = "original", narrative = FALSE, progress = TRUE, ...){
 
   #### Class Conversion and Argument Checks ####
   convert <- tomatrix(B)
@@ -86,17 +84,6 @@ osdsm <- function(B, alpha = 0.05, trials = NULL, signed = FALSE, mtc = "none",
   #### Bipartite Projection ####
   P <- tcrossprod(B)
 
-  #### Compute number of trials needed ####
-  if (is.null(trials)) {
-    trials.alpha <- alpha
-    if (signed == TRUE) {trials.alpha <- trials.alpha / 2}  #Two-tailed test
-    if (mtc != "none") {  #Adjust trial.alpha using Bonferroni
-      if (signed == TRUE) {trials.alpha <- trials.alpha / ((nrow(B)*(nrow(B)-1))/2)}  #Every edge must be tested
-      if (signed == FALSE) {trials.alpha <- trials.alpha / (sum(P>0)/2)}  #Every non-zero edge in the projection must be tested
-    }
-    trials <- ceiling((stats::power.prop.test(p1 = trials.alpha * 0.95, p2 = trials.alpha, sig.level = alpha, power = (1-alpha), alternative = "one.sided")$n)/2)
-  }
-
   ### Create Positive and Negative Matrices to hold backbone ###
   Pupper <- matrix(0, nrow(P), ncol(P))
   if (signed) {Plower <- matrix(0, nrow(P), ncol(P))}
@@ -127,6 +114,7 @@ osdsm <- function(B, alpha = 0.05, trials = NULL, signed = FALSE, mtc = "none",
   A$rand <- NA
 
   #### Build null models ####
+  if (is.null(trials)) {trials <- trials.needed(M = P, alpha = alpha, signed = signed, missing.as.zero = missing.as.zero, mtc = mtc, ...)}
   if (progress) {message(paste0("Constructing empirical edgewise p-values using ", trials, " trials -" ))}
   if (progress) {pb <- utils::txtProgressBar(min = 0, max = trials, style = 3)}  #Start progress bar
   for (i in 1:trials){
@@ -150,6 +138,12 @@ osdsm <- function(B, alpha = 0.05, trials = NULL, signed = FALSE, mtc = "none",
   Pupper <- (Pupper/trials)
   if (signed) {Plower <- (Plower/trials)}
 
+  #### If missing edges should *not* be treated as having zero weight, remove p-value and do not consider for backbone ####
+  if (!missing.as.zero) {
+    Pupper[P == 0] <- NA
+    if (signed) {Plower[P == 0] <- NA}
+  }
+
   #### Create Backbone Object ####
   bb <- list(G = P,  #Preliminary backbone object
              Pupper = Pupper,
